@@ -113,6 +113,8 @@ static void   theme_subdir_load            (GnomeIconLoader      *loader,
 static void   icon_data_destroy            (GnomeIconData        *data);
 static void   blow_themes                  (GnomeIconLoaderPrivate *priv);
 
+static IconSuffix suffix_from_name         (const char           *name);
+
 
 static guint		 signal_changed = 0;
 
@@ -220,6 +222,8 @@ setup_gconf_handler (GnomeIconLoader *loader)
       g_free (priv->current_theme);
       priv->current_theme = theme;
     }
+
+  g_object_unref (client);
 }
 
 static void
@@ -227,17 +231,18 @@ remove_gconf_handler (GnomeIconLoader *loader)
 {
   GnomeIconLoaderPrivate *priv;
   GConfClient *client;
-  char *theme;
   
   priv = loader->priv;
 
   g_assert (priv->theme_changed_id != 0);
-  
-  gconf_client_notify_remove (gconf_client_get_default (),
-			      priv->theme_changed_id);
-  priv->theme_changed_id = 0;
-}
 
+  client = gconf_client_get_default ();
+  
+  gconf_client_notify_remove (client, priv->theme_changed_id);
+  priv->theme_changed_id = 0;
+
+  g_object_unref (client);
+}
 
 static void
 gnome_icon_loader_init (GnomeIconLoader *loader)
@@ -250,15 +255,17 @@ gnome_icon_loader_init (GnomeIconLoader *loader)
 
   priv->custom_theme = FALSE;
   priv->current_theme = g_strdup ("default");
-  priv->search_path = g_new (char *, 3);
+  priv->search_path = g_new (char *, 5);
   
 
   priv->search_path[0] = g_build_filename (g_get_home_dir (),
 					     ".icons",
 					     NULL);
-  priv->search_path[1] = g_strdup ("/usr/share/icons");
-  priv->search_path[2] = g_strdup ("/usr/share/pixmaps");
-  priv->search_path_len = 3;
+  priv->search_path[1] = g_strdup (GNOME_DESKTOP_PIXMAPDIR);
+  priv->search_path[2] = g_strdup (GNOME_DESKTOP_ICONDIR);
+  priv->search_path[3] = g_strdup ("/usr/share/icons");
+  priv->search_path[4] = g_strdup ("/usr/share/pixmaps");
+  priv->search_path_len = 5;
 
   priv->themes_valid = FALSE;
   priv->themes = NULL;
@@ -303,11 +310,7 @@ gnome_icon_loader_finalize (GObject *object)
   priv->search_path = NULL;
 
   if (priv->theme_changed_id)
-    {
-      gconf_client_notify_remove (gconf_client_get_default (),
-				  priv->theme_changed_id);
-      priv->theme_changed_id = 0;
-    }
+    remove_gconf_handler (loader);
 
   blow_themes (priv);
 
@@ -574,6 +577,8 @@ load_themes (GnomeIconLoader *loader)
   char *dir, *base_name, *dot;
   const char *file;
   char *abs_file;
+  char *old_file;
+  IconSuffix old_suffix, new_suffix;
   
   priv = loader->priv;
 
@@ -598,20 +603,29 @@ load_themes (GnomeIconLoader *loader)
 	      my_g_str_has_suffix (file, ".xpm"))
 	    {
 	      abs_file = g_build_filename (dir, file, NULL);
-	      if (g_file_test (abs_file, G_FILE_TEST_IS_REGULAR))
+
+	      base_name = g_strdup (file);
+		  
+	      dot = strrchr (base_name, '.');
+	      if (dot)
+		*dot = 0;
+
+	      if ((old_file = g_hash_table_lookup (priv->unthemed_icons,
+						   base_name)))
 		{
-		  base_name = g_strdup (file);
-		  
-		  dot = strrchr (base_name, '.');
-		  if (dot)
-		    *dot = 0;
-		  
-		  g_hash_table_replace (priv->unthemed_icons,
-					base_name,
-					abs_file);
+		  old_suffix = suffix_from_name (old_file);
+		  new_suffix = suffix_from_name (file);
+
+		  if (new_suffix > old_suffix)
+		    g_hash_table_replace (priv->unthemed_icons,
+					  base_name,
+					  abs_file);
+
 		}
 	      else
-		g_free (abs_file);
+		g_hash_table_insert (priv->unthemed_icons,
+				     base_name,
+				     abs_file);
 	    }
 	}
       g_dir_close (gdir);
@@ -829,6 +843,23 @@ string_from_suffix (IconSuffix suffix)
   return NULL;
 }
 
+static IconSuffix
+suffix_from_name (const char *name)
+{
+  IconSuffix retval;
+
+  if (my_g_str_has_suffix (name, ".png"))
+    retval = ICON_SUFFIX_PNG;
+  else if (my_g_str_has_suffix (name, ".svg"))
+    retval = ICON_SUFFIX_SVG;
+  else if (my_g_str_has_suffix (name, ".xpm"))
+    retval = ICON_SUFFIX_XPM;
+  else
+    retval = ICON_SUFFIX_NONE;
+
+  return retval;
+}
+
 static char *
 theme_lookup_icon (IconTheme *theme,
 		   const char *icon_name,
@@ -1042,16 +1073,12 @@ scan_directory (IconThemeDir *dir, char *full_dir, gboolean allow_svg)
 	  
 	  continue;
 	}
-      
-      if (my_g_str_has_suffix (name, ".png")) 
-	suffix = ICON_SUFFIX_PNG;
-      else if (allow_svg && my_g_str_has_suffix (name, ".svg")) 
-	suffix = ICON_SUFFIX_SVG;
-      else if (my_g_str_has_suffix (name, ".xpm"))
-	suffix = ICON_SUFFIX_XPM;
-      else
-	continue;
 
+      suffix = suffix_from_name (name);
+      if (suffix == ICON_SUFFIX_NONE ||
+	  (suffix == ICON_SUFFIX_SVG && !allow_svg))
+	continue;
+      
       base_name = g_strdup (name);
       dot = strrchr (base_name, '.');
       *dot = 0;
